from typing import List, Tuple

import openai

preprompt = {
    'en': """The following is a conversation with an AI assistant. The assistant is helpful, creative, and clever.
""",
    'no': """Det følgende er en samtale med en AI-assistent. Assistenten er hjelpsom, kreativ, smart og oppriktig.
""",
}
query_prefix = {"en": "\nHuman: ", "no": "\nMenneske: "}
query_suffix = {"en": "\nAI: ", "no": "\nAI: "}
stop = {
    "en": ["Human:", "AI:"],
    "no": ["Menneske:", "AI:"]
}

openai.api_key = input("Enter your OpenAI API key: ").strip()
models = [
    "text-davinci-003",  # Most capable GPT-3 model. Can do any task the other models can do ...
    "text-curie-001",  # Very capable, but faster and lower cost than Davinci
    "text-babbage-001",  # Capable of straightforward tasks, very fast, and lower cost.
    "text-ada-001",  # Capable of very simple tasks, usually the fastest model in the GPT-3 series, and lowest cost.
]
current_model = models[0]


def respond(conversation: List[Tuple[str]], query, language):
    return respond_inner(
        conversation, query,
        preprompt[language], query_prefix[language], query_suffix[language], stop[language]
    )


def respond_inner(conversation: List[Tuple[str]], query, preprompt, query_prefix, query_suffix, stop):
    global current_model
    prompt = preprompt

    for turn in conversation:
        prompt += query_prefix + turn[0] + query_suffix + turn[1]

    prompt += query_prefix + query + query_suffix
    print(prompt)

    if len(prompt) > 512:
        del conversation[0]
        print("Conversation is too long, will delete oldest turn and retry")
        return respond_inner(conversation, query, preprompt, query_prefix, query_suffix, stop)

    try:
        response = openai.Completion.create(
            model=current_model,
            prompt=prompt,
            max_tokens=200,
            temperature=0,
            stop=stop,
            timeout=10,
        )
    except Exception as e:
        print("Got an error, switching away from", current_model)
        current_model = models[(models.index(current_model) + 1) % len(models)]
        print("Next request will use", current_model)
        raise e

    print(response)
    actual_response = response['choices'][0]
    text = actual_response['text']
    if actual_response['finish_reason'] != 'stop':
        print("Strange stop:", actual_response)
        text += "..."
    print(text)
    try:
        stop_at = text.index(query_prefix)
        print("Stop after char", stop_at)
        return text[:stop_at].strip()
    except ValueError:
        print("No query_prefix found, return everything")
        return text.strip()
